
mon.inf.lag.1p.nll = function(nI, tps, r, t0, Imax, steps = 0.1, tracing = 0) {
	if(r <= 0 || t0 <= 0 || t0 >= min(tps[nI>0])){ return(Inf) }
	Isim = sim.inf.1p(tps, r, Imax, t0, steps)
	negLL = -sum(dbinom(x = nI, prob =  Isim / Imax, size = Imax, log = TRUE))
	if(tracing == 1) print(paste("negLL: ", negLL, "r: ",r, "t0: ", t0 ))
	return(negLL)
}

mon.inf.lag.2p.nll = function(nI, tps, rp, t0p, rc, t0c, Imax, steps = 0.1, tracing = 0) {
	if(rp <= 0 || t0p <= 0 || rc <= 0 || t0c <= 0){ return(Inf) }
	if(t0p >= min(tps[nI>0]) && t0c >= min(tps[nI>0])){ return(Inf) }
	Isim = sim.inf.2p(tps, rp, rc, Imax, t0p, t0c, steps)
	negLL = -sum(dbinom(x = nI, prob = Isim / Imax, size = Imax, log = TRUE))
	if(tracing == 1) print(paste("negLL: ", negLL, "rp: ",rp, "t0p: ", t0p ))
	return(negLL)
}

sim.inf.1p = function(tps, r, Imax, t0, steps) {
	Iout = numeric()
	if(length(Imax) == 1){
		mres = data.frame(lsoda(y = c(I = 0),
					times = sort(unique(c(seq(0, max(tps), steps), tps))),
					func = mon.inf.lag.1p,
					parms = c(r = r,
						Imax = Imax,
						t0 = t0)))
		for(tps.i in 1:length(tps)){
			Iout[tps.i] = mres$I[mres$time == tps[tps.i]]
		}
	} else {
		mres.I = numeric()
		mres.time = numeric()
		mres.Imax = numeric()
		for(Imax.i in 1:length(unique(Imax))){
			mres = data.frame(lsoda(y = c(I = 0),
					times = sort(unique(c(seq(0, max(tps), steps), tps))),
					func = mon.inf.lag.1p,
					parms = c(r = r,
						Imax = unique(Imax)[Imax.i],
						t0 = t0)))
			mres.I = c(mres.I,mres$I)
			mres.time = c(mres.time,mres$time)
			mres.Imax = c(mres.Imax,rep(unique(Imax)[Imax.i],length(mres$time)))
			}
		for(tps.i in 1:length(tps)){
			Iout[tps.i] = mres.I[mres.time == tps[tps.i] & mres.Imax == Imax[tps.i]]
		}
	}
	return(Iout)
}

sim.inf.2p = function(tps, rp, rc, Imax, t0p, t0c, steps) {
	Iout = numeric()
	if(length(Imax) == 1){
		mres = data.frame(lsoda(y = c(Ip = 0, Ic = 0),
			times = sort(unique(c(seq(0, max(tps), steps), tps))),
			func = mon.inf.lag.2p,
			parms = c(rp = rp,
				rc = rc,
				Imax = Imax,
				t0p = t0p,
				t0c = t0c)))
		for(tps.i in 1:length(tps)){
			Iout[tps.i] = (mres$Ip+mres$Ic)[mres$time == tps[tps.i]]
		}
	} else {
		mres.I = numeric()
		mres.time = numeric()
		mres.Imax = numeric()
		for(Imax.i in 1:length(unique(Imax))){
			mres = data.frame(lsoda(y = c(Ip = 0, Ic = 0),
				times = sort(unique(c(seq(0, max(tps), steps), tps))),
				func = mon.inf.lag.2p,
				parms = c(rp = rp,
					rc = rc,
					Imax = unique(Imax)[Imax.i],
					t0p = t0p,
					t0c = t0c)))
			mres.I = c(mres.I, mres$Ip+mres$Ic)
			mres.time = c(mres.time, mres$time)
			mres.Imax = c(mres.Imax, rep(unique(Imax)[Imax.i],length(mres$time)))
		}
		for(tps.i in 1:length(tps)){
			Iout[tps.i] = mres.I[mres.time == tps[tps.i] & mres.Imax == Imax[tps.i]]
		}
	}
	return(Iout)
}
